import argparse
import json
import yaml
import os
import logging
from collections import OrderedDict
from glu.util import OrderedDictYamlLoader

from glu import create_scope


def load_file(path):
    if path.lower().endswith('.yml') or path.lower().endswith('.yaml'):
        with open(path, 'r') as f:
            return yaml.load(f, Loader=OrderedDictYamlLoader)

    else:
        with open(path, 'r') as f:
            return json.load(f, object_pairs_hook=OrderedDict)


def load_scope_from_file(scope, file_uri):
    path = file_uri
    params = {}
    if '?' in file_uri:
        path, params = file_uri.split('?')
        params = dict(tuple(field_and_value.split('='))
                      for field_and_value in params.split('&'))

    scope.load(load_file(path), **params)


def parse():
    parser = argparse.ArgumentParser()
    parser.add_argument('-s', '--source', required=True, dest='sources', nargs='+', help='Path to the source file')
    parser.add_argument('-e', '--use-env', action='store_true', help='Use environment variable')
    parser.add_argument('-t', '--template', required=True, help='Path to the template file')
    parser.add_argument('-o', '--output', required=True, help='Path for output file')
    return parser.parse_args()


def main():
    args = parse()
    scope = create_scope()

    if args.use_env:
        scope.load(dict(os.environ.items()), load_to='@env')

    for file_uri in args.sources:
        load_scope_from_file(scope, file_uri)

    template = load_file(args.template)
    res = scope.glue(template)

    logging.info('Result:\n{}'.format(json.dumps(res, indent=2)))

    with open(args.output, 'w') as f:
        json.dump(res, f, indent=2, separators=(',', ': '))
        f.write('\n')



import re
import uuid
from datetime import datetime
from future.utils import iteritems
from past.types import basestring
from glu.util import head_tail, get_dot, set_dot_default
from glu.filter import apply_filters, undefined, muted
import time


class Scope(object):
    def __init__(self):
        self.scope = {}

    def reset(self, set_basic=True):
        self.scope.clear()
        if set_basic:
            self.scope['@date'] = datetime.today().strftime('%Y%m%d')
            self.scope['@timestamp'] = int(time.time())
            for i in range(10):
                self.scope['@uuid_{}'.format(i)] = str(uuid.uuid4())

    def load(self, scope, load_from=None, load_to=None):
        scope = scope if load_from is None \
            else get_dot(scope, load_from, {})
        load_point = self.scope if load_to is None \
            else set_dot_default(self.scope, load_to, {})
        load_point.update(scope)

    def copy(self):
        scope = Scope()
        scope.scope = self.scope.copy()
        return scope

    def override(self, *scope_keys):
        scope = self.copy()
        for s in scope_keys:
            if get_dot(scope.scope, s) is None:
                raise KeyError('Undefined scope key: {}'.format(s))
            scope.load(get_dot(scope.scope, s))
        return scope

    def resolve(self, key):
        res = get_dot(self.scope, key, undefined)
        if res is undefined:
            return undefined
        return self.glue(res)

    def evaluate(self, expr):
        statement, filters = head_tail(x.strip() for x in expr.split('|'))
        clauses = [x.strip() for x in statement.split('??')]

        res = undefined
        for clause in clauses:
            key, scopes = head_tail(x.strip() for x in clause.split('<'))
            res = (self if len(scopes) == 0
                   else self.override(*scopes)).resolve(key)
            if res is not undefined:
                break

        res = apply_filters(res, *reversed(filters))
        if res is undefined:
            last_key = clauses[-1].split('<')[0].strip()
            raise KeyError('Cannot find key {}'.format(last_key))
        return res

    def interpolate(self, pattern):
        rendered = pattern
        for template in re.findall('{{[^{}]+?}}', pattern):
            expr = template[2:-2].strip()
            res = self.evaluate(expr)
            assert isinstance(res, (int, basestring)), (
                'Cannot interpolate {} (={});'
                ' Only int or str value is allowed').format(template, res)
            rendered = rendered.replace(template, str(res))
        return rendered

    def glue(self, value):
        if isinstance(value, list):
            return [self.glue(item) for item in value
                    if item is not muted]
        if isinstance(value, dict):
            return type(value)([kv for kv in [(k, self.glue(v)) for k, v in iteritems(value)]
                                if kv[1] is not muted])
        if isinstance(value, basestring):
            match = re.match('^{{([^{}]+)}}$', value)
            if match is not None:
                return self.evaluate(match.group(1))
            return self.interpolate(value)
        return value


def create_scope(set_basic=True):
    scope = Scope()
    scope.reset(set_basic=set_basic)
    return scope



import re
from functools import wraps
from glu.util import head_tail

undefined = object()
muted = object()
filters = {}


def ensure_string(string):
    assert isinstance(string, (int, str)), 'Cannot apply filter to non-string value {}'.format(string)
    return str(string)


def string_filter(func):
    @wraps(func)
    def wrapper(string):
        if string is undefined or string is muted:
            return string
        return func(ensure_string(string))

    filters[func.__name__] = wrapper
    return wrapper


def mute_filter(cond):
    @wraps(cond)
    def wrapper(variable):
        return muted if cond(variable) else variable

    filters[cond.__name__] = wrapper
    return wrapper


@string_filter
def tokenize(string):
    string = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', string)
    string = re.sub('([a-z0-9])([A-Z])', r'\1_\2', string)
    return re.compile('[\s_-]+').split(string)


def capitalize(string):
    return string[0].upper() + string[1:].lower()


@string_filter
def display(string):
    return ' '.join(capitalize(token) for token in tokenize(string))


@string_filter
def pascal_case(string):
    return ''.join(capitalize(token) for token in tokenize(string))


@string_filter
def camel_case(string):
    pascal = pascal_case(string)
    return pascal[0].lower() + pascal[1:]


@string_filter
def snake_case(string):
    return '_'.join(token.lower() for token in tokenize(string))


@string_filter
def lisp_case(string):
    return '-'.join(token.lower() for token in tokenize(string))


@string_filter
def uppercase(string):
    return '_'.join(token.upper() for token in tokenize(string))


@string_filter
def uppercase_dash(string):
    return '-'.join(token.upper() for token in tokenize(string))


@string_filter
def capitalize_dash(string):
    return '-'.join(capitalize(token) for token in tokenize(string))


@mute_filter
def mute(variable):
    return variable is undefined


@mute_filter
def mute_if_undefined(variable):
    return variable is undefined


@mute_filter
def mute_if_empty(variable):
    if variable is undefined:
        return True
    if hasattr(variable, '__len__'):
        return len(variable) == 0
    return False


def apply_filters(item, *exprs):
    for expr in exprs:
        key, args = head_tail(expr.split())
        assert key in filters, 'Unknown filter {}'.format(key)
        item = filters[key](item, *args)

    return item



from glu import create_scope
import unittest


class CoreTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.scope = create_scope(set_basic=False)
        
    def setUp(self):
        self.scope.reset(set_basic=False)

    def test_interpolation(self):
        self.scope.load({
            'name': 'World'
        })

        self.assertEqual(
            self.scope.glue('hello, {{ name }}'),
            'hello, World')

    def test_interpolation_fail(self):
        self.scope.load({
            'int_value': 123,
            'array_value': [1, 2, 3],
            'dict_value': {'foo': 'bar'}
        })

        self.assertEqual(
            self.scope.glue('int: {{ int_value }}'),
            'int: 123')

        with self.assertRaises(AssertionError):
            self.scope.glue('array: {{ array_value }}')

        with self.assertRaises(AssertionError):
            self.scope.glue('dict: {{ dict_value }}')

    def test_load_from_to(self):
        self.scope.load({
            'score': {
                'math': 93
            }
        }, load_from='score', load_to='midterm_score')

        self.assertEqual(self.scope.glue('{{ midterm_score.math }}'), 93)

    def test_fallback(self):
        self.scope.load({
            'score': {
                'math': 93
            }
        })

        self.assertEqual(
            self.scope.glue('{{ score.science ?? score.history ?? score.math }}'), 93)

    def test_string_filters(self):
        self.scope.load({
            'foo': 'This_isREALLYAmazing'
        })

        self.assertEqual(
            self.scope.glue({
                'display': '{{ foo | display }}',
                'camel_case': '{{ foo | camel_case }}',
                'pascal_case': '{{ foo | pascal_case }}',
                'snake_case': '{{ foo | snake_case }}',
                'lisp_case': '{{ foo | lisp_case }}',
                'uppercase': '{{ foo | uppercase }}',
                'uppercase_dash': '{{ foo | uppercase_dash }}',
                'capitalize_dash': '{{ foo | capitalize_dash }}',
            }),
            {
                'display': 'This Is Really Amazing',
                'camel_case': 'thisIsReallyAmazing',
                'pascal_case': 'ThisIsReallyAmazing',
                'snake_case': 'this_is_really_amazing',
                'lisp_case': 'this-is-really-amazing',
                'uppercase': 'THIS_IS_REALLY_AMAZING',
                'uppercase_dash': 'THIS-IS-REALLY-AMAZING',
                'capitalize_dash': 'This-Is-Really-Amazing',
            }
        )

    def test_mute(self):
        self.scope.load({
            'empty_array': [],
            'empty_string': '',
            'empty_dict': {}
        })

        self.assertEqual(
            self.scope.glue({
                'foo': '{{ empty_array | mute_if_empty }}',
                'bar': '{{ empty_string | mute_if_empty }}',
                'zoo': '{{ empty_dict | mute_if_empty }}',
                'coz': '{{ undefined_var | mute }}'
            }), {})

    def test_scope(self):
        self.scope.load({
            'greeting': 'hello, {{ name }}'
        })
        self.scope.load({
            'foo_scope': {
                'name': 'foo'
            },
            'bar_scope': {
                'name': 'bar'
            }
        })

        self.assertEqual(
            self.scope.glue({
                'hello_foo': '{{ greeting < foo_scope }}',
                'hello_bar': '{{ greeting < bar_scope }}'
            }), {
                'hello_foo': 'hello, foo',
                'hello_bar': 'hello, bar'
            })



from setuptools import setup, find_packages

setup(
    name='glu',
    packages=find_packages(),
    url='https://github.com/chongkong/glu',
    license='MIT',
    version='0.0.17',
    description='Glue for DRY configurations',
    author='Park Jong Bin',
    author_email='chongkong94@gmail.com',
    keywords=['glue', 'glu', 'dry', 'config', 'dry-config', 'dry-configurable'],
    zip_safe=False,
    install_requires=[
        'future==0.15.2',
        'PyYAML==3.11'
    ],
    entry_points={
        'console_scripts': [
            'glu = glu.cli:main'
        ]
    },
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ]
)



import yaml
from collections import OrderedDict


class OrderedDictYamlLoader(yaml.Loader):
    pass


def construct_mapping(loader, node):
    loader.flatten_mapping(node)
    return OrderedDict(loader.construct_pairs(node))

OrderedDictYamlLoader.add_constructor(
    yaml.resolver.BaseResolver.DEFAULT_MAPPING_TAG,
    construct_mapping)


def head_tail(iterable):
    """ For python2 compatibility """
    lst = list(iterable)
    return lst[0], lst[1:]


def tail_head(iterable):
    """ For python2 compatibility """
    lst = list(iterable)
    return lst[:-1], lst[-1]


def get_dot(dic, key, default=None):
    """ Similar to dict.get(), but key is given with dot (eg. foo.bar) and
    result is evaluated in generous way. That is, get_dot(dic, 'foo.bar.vaz')
    will return dic['foo']['bar'] if both dic['foo'] and dic['foo']['baz'] exists,
    but return default if any of them does not exists.
    """
    keys = key.split('.')
    res = dic
    for k in keys:
        if not isinstance(res, dict):
            return default
        elif k in res:
            res = res[k]
        else:
            return default
    return res


def set_dot_default(dic, key, value):
    """ Similar to dict.set_default(), but key is given with dot (eg. foo.bar)
    Refer docs of get_dot() for more detail. """
    keys, last_key = tail_head(key.split('.'))
    res = dic
    for k in keys:
        if not isinstance(res, dict):
            raise ValueError('Cannot access {}: {} is not a dictionary'.format(key, k))
        res.setdefault(k, {})
        res = res[k]
    if last_key not in res:
        res[last_key] = value
    return res[last_key]


if __name__ == '__main__':
    dic = {}
    set_dot_default(dic, 'a.b.c.d', 'hello')
    hello = get_dot(dic, 'a.b.c.d')
    assert hello == 'hello', 'hello != {}'.format(hello)
